@@ -1,6 +1,6 @@
 /* -*- mode:C; c-file-style: "bsd" -*- */
 /*
- * Copyright (c) 2008, 2009, Yubico AB
+ * Copyright (c) 2008, 2009, 2010, 2011 Yubico AB
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,6 +38,7 @@
 #include <string.h>
 #include <time.h>
 #include <ctype.h>
+#include <assert.h>
 
 #include <yubikey.h>
 
@@ -55,7 +56,7 @@ static const YK_CONFIG default_config1 = {
 	{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* key */
 	{ 0, 0, 0, 0, 0, 0 },	/* accCode */
 	0,			/* fixedSize */
-	0,			/* pgmSeq */
+	0,			/* extFlags */
 	TKTFLAG_APPEND_CR,	/* tktFlags */
 	0,			/* cfgFlags */
 	0,			/* ctrOffs */
@@ -68,7 +69,7 @@ static const YK_CONFIG default_config2 = {
 	{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* key */
 	{ 0, 0, 0, 0, 0, 0 },	/* accCode */
 	0,			/* fixedSize */
-	0,			/* pgmSeq */
+	0,			/* extFlags */
 	TKTFLAG_APPEND_CR,	/* tktFlags */
 	/* cfgFlags */
 	CFGFLAG_STATIC_TICKET | CFGFLAG_STRONG_PW1 | CFGFLAG_STRONG_PW2 | CFGFLAG_MAN_UPDATE,
@@ -126,23 +127,39 @@ int ykp_configure_for(YKP_CONFIG *cfg, int confnum, YK_STATUS *st)
 	return 0;
 }
 
+/* Return number of bytes of key data for this configuration.
+ * 20 bytes is 160 bits, 16 bytes is 128.
+ */
+int _get_supported_key_length(const YKP_CONFIG *cfg)
+{
+	bool key_bits_in_uid = false;
+
+	/* OATH-HOTP and HMAC-SHA1 challenge response support 20 byte (160 bits)
+	 * keys, holding the last four bytes in the uid field.
+	 */
+	if ((cfg->ykcore_config.tktFlags & TKTFLAG_OATH_HOTP) == TKTFLAG_OATH_HOTP)
+		return 20;
+
+	if ((cfg->ykcore_config.tktFlags & TKTFLAG_CHAL_RESP) == TKTFLAG_CHAL_RESP &&
+	    (cfg->ykcore_config.cfgFlags & CFGFLAG_CHAL_HMAC) == CFGFLAG_CHAL_HMAC) {
+		return 20;
+	}
+
+	return 16;
+}
+
+/* Decode 128 bit AES key into cfg->ykcore_config.key */
 int ykp_AES_key_from_hex(YKP_CONFIG *cfg, const char *hexkey) {
 	char aesbin[256];
 
-/* Make sure that the hexkey is exactly 32 characters */
+	/* Make sure that the hexkey is exactly 32 characters */
 	if (strlen(hexkey) != 32) {
 		return 1;  /* Bad AES key */
 	}
 
-/* Make sure that the hexkey is made up of only [0-9a-f] */
-	int i;
-	for (i=0; i < strlen(hexkey); i++) {
-		char c = tolower(hexkey[i]);
-/* In ASCII, 0-9 == 48-57 and a-f == 97-102 */
-		if ( c<48 || (c>57 && c<97) || c>102 ) {
-			return 1;
-		}
-	}
+	/* Make sure that the hexkey is made up of only [0-9a-f] */
+	if (! yubikey_hex_p(hexkey))
+		return 1;
 
 	yubikey_hex_decode(aesbin, hexkey, sizeof(aesbin));
 	memcpy(cfg->ykcore_config.key, aesbin, sizeof(cfg->ykcore_config.key));
@@ -150,6 +167,39 @@ int ykp_AES_key_from_hex(YKP_CONFIG *cfg, const char *hexkey) {
 	return 0;
 }
 
+/* Decode 160 bits HMAC key, used with OATH and HMAC challenge-response.
+ *
+ * The first 128 bits of the HMAC go key into cfg->ykcore_config.key,
+ * and 32 bits into the first four bytes of cfg->ykcore_config.uid.
+*/
+int ykp_HMAC_key_from_hex(YKP_CONFIG *cfg, const char *hexkey) {
+	char aesbin[256];
+	int i;
+
+	/* Make sure that the hexkey is exactly 40 characters */
+	if (strlen(hexkey) != 40) {
+		return 1;  /* Bad HMAC key */
+	}
+
+	/* Make sure that the hexkey is made up of only [0-9a-f] */
+	if (! yubikey_hex_p(hexkey))
+		return 1;
+
+	yubikey_hex_decode(aesbin, hexkey, sizeof(aesbin));
+	i = sizeof(cfg->ykcore_config.key);
+	memcpy(cfg->ykcore_config.key, aesbin, i);
+	memcpy(cfg->ykcore_config.uid, aesbin + i, 20 - i);
+
+	return 0;
+}
+
+/* Generate an AES (128 bits) or HMAC (despite the function name) (160 bits)
+ * key from user entered input.
+ *
+ * Use user provided salt, or use salt from an available random device.
+ * If no random device is available we fall back to using 2048 bits of
+ * system time data, together with the user input, as salt.
+ */
 int ykp_AES_key_from_passphrase(YKP_CONFIG *cfg, const char *passphrase,
 				const char *salt)
 {
@@ -163,6 +213,11 @@ int ykp_AES_key_from_passphrase(YKP_CONFIG *cfg, const char *passphrase,
 		char **random_place;
 		uint8_t _salt[8];
 		size_t _salt_len = 0;
+		unsigned char buf[sizeof(cfg->ykcore_config.key) + 4];
+		int rc;
+		int key_bytes = _get_supported_key_length(cfg);
+
+		assert (key_bytes <= sizeof(buf));
 
 		if (salt) {
 			_salt_len = strlen(salt);
@@ -178,8 +233,8 @@ int ykp_AES_key_from_passphrase(YKP_CONFIG *cfg, const char *passphrase,
 					size_t read_bytes = 0;
 
 					while (read_bytes < sizeof(_salt)) {
-						size_t n = fread(&cfg->ykcore_config.key[read_bytes],
-								 1, KEY_SIZE - read_bytes,
+						size_t n = fread(&_salt[read_bytes],
+								 1, sizeof (_salt) - read_bytes,
 								 random_file);
 						read_bytes += n;
 					}
@@ -207,11 +262,22 @@ int ykp_AES_key_from_passphrase(YKP_CONFIG *cfg, const char *passphrase,
 			_salt_len = sizeof(_salt);
 		}
 
-		return yk_pbkdf2(passphrase,
-				 _salt, _salt_len,
-				 1024,
-				 cfg->ykcore_config.key, sizeof(cfg->ykcore_config.key),
-				 &yk_hmac_sha1);
+		rc = yk_pbkdf2(passphrase,
+			       _salt, _salt_len,
+			       1024,
+			       buf, key_bytes,
+			       &yk_hmac_sha1);
+
+		if (rc) {
+			memcpy(cfg->ykcore_config.key, buf, sizeof(cfg->ykcore_config.key));
+
+			if (key_bytes == 20) {
+				memcpy(cfg->ykcore_config.uid, buf + sizeof(cfg->ykcore_config.key), 4);
+			}
+		}
+
+		memset (buf, 0, sizeof(buf));
+		return rc;
 	}
 	return 0;
 }
@@ -236,6 +302,13 @@ static bool vcheck_v21_or_greater(const YKP_CONFIG *cfg)
 		cfg->yk_major_version > 2;
 }
 
+static bool vcheck_v22_or_greater(const YKP_CONFIG *cfg)
+{
+	return (cfg->yk_major_version == 2 &&
+		cfg->yk_minor_version >= 2) ||
+		cfg->yk_major_version > 2;
+}
+
 #define def_set_charfield(fnname,fieldname,size,extra,vcheck)	\
 int ykp_set_ ## fnname(YKP_CONFIG *cfg, unsigned char *input, size_t len)	\
 {								\
@@ -300,6 +373,24 @@ int ykp_set_cfgflag_ ## type(YKP_CONFIG *cfg, bool state)		\
 	return 0;						\
 }
 
+#define def_set_extflag(type,vcheck)				\
+int ykp_set_extflag_ ## type(YKP_CONFIG *cfg, bool state)		\
+{								\
+	if (cfg) {						\
+		if (!vcheck(cfg)) {				\
+			ykp_errno = YKP_EYUBIKEYVER;		\
+			return 0;				\
+		}						\
+		if (state)					\
+			cfg->ykcore_config.extFlags |= EXTFLAG_ ## type;	\
+		else						\
+			cfg->ykcore_config.extFlags &= ~EXTFLAG_ ## type;	\
+		return 1;					\
+	}							\
+	ykp_errno = YKP_ENOCFG;					\
+	return 0;						\
+}
+
 def_set_tktflag(TAB_FIRST,vcheck_all)
 def_set_tktflag(APPEND_TAB1,vcheck_all)
 def_set_tktflag(APPEND_TAB2,vcheck_all)
@@ -308,6 +399,7 @@ def_set_tktflag(APPEND_DELAY2,vcheck_all)
 def_set_tktflag(APPEND_CR,vcheck_all)
 def_set_tktflag(PROTECT_CFG2,vcheck_no_v1)
 def_set_tktflag(OATH_HOTP,vcheck_v21_or_greater)
+def_set_tktflag(CHAL_RESP,vcheck_v22_or_greater)
 
 def_set_cfgflag(SEND_REF,vcheck_all)
 def_set_cfgflag(TICKET_FIRST,vcheck_v1)
@@ -323,6 +415,14 @@ def_set_cfgflag(OATH_HOTP8,vcheck_v21_or_greater)
 def_set_cfgflag(OATH_FIXED_MODHEX1,vcheck_v21_or_greater)
 def_set_cfgflag(OATH_FIXED_MODHEX2,vcheck_v21_or_greater)
 def_set_cfgflag(OATH_FIXED_MODHEX,vcheck_v21_or_greater)
+def_set_cfgflag(CHAL_YUBICO,vcheck_v22_or_greater)
+def_set_cfgflag(CHAL_HMAC,vcheck_v22_or_greater)
+def_set_cfgflag(HMAC_LT64,vcheck_v22_or_greater)
+def_set_cfgflag(CHAL_BTN_TRIG,vcheck_v22_or_greater)
+
+def_set_extflag(SERIAL_BTN_VISIBLE,vcheck_v22_or_greater)
+def_set_extflag(SERIAL_USB_VISIBLE,vcheck_v22_or_greater)
+def_set_extflag(SERIAL_API_VISIBLE,vcheck_v22_or_greater)
 
 const char str_key_value_separator[] = ": ";
 const char str_hex_prefix[] = "h:";
@@ -338,40 +438,65 @@ struct map_st {
 	uint8_t flag;
 	const char *flag_text;
 	bool (*vcheck)(const YKP_CONFIG *cfg);
+	unsigned char tkt_context;
 };
 
 const char str_ticket_flags[] = "ticket_flags";
 struct map_st ticket_flags_map[] = {
-	{ TKTFLAG_TAB_FIRST, "TAB_FIRST", vcheck_all },
-	{ TKTFLAG_APPEND_TAB1, "APPEND_TAB1", vcheck_all },
-	{ TKTFLAG_APPEND_TAB2, "APPEND_TAB2", vcheck_all },
-	{ TKTFLAG_APPEND_DELAY1, "APPEND_DELAY1", vcheck_all },
-	{ TKTFLAG_APPEND_DELAY2, "APPEND_DELAY2", vcheck_all },
-	{ TKTFLAG_APPEND_CR, "APPEND_CR", vcheck_all },
-	{ TKTFLAG_PROTECT_CFG2, "PROTECT_CFG2", vcheck_no_v1 },
-	{ TKTFLAG_OATH_HOTP, "OATH_HOTP", vcheck_v21_or_greater },
-	{ 0, "" }
+	{ TKTFLAG_TAB_FIRST,		"TAB_FIRST",		vcheck_all,		0 },
+	{ TKTFLAG_APPEND_TAB1,		"APPEND_TAB1",		vcheck_all,		0 },
+	{ TKTFLAG_APPEND_TAB2,		"APPEND_TAB2",		vcheck_all,		0 },
+	{ TKTFLAG_APPEND_DELAY1,	"APPEND_DELAY1",	vcheck_all,		0 },
+	{ TKTFLAG_APPEND_DELAY2,	"APPEND_DELAY2",	vcheck_all,		0 },
+	{ TKTFLAG_APPEND_CR,		"APPEND_CR",		vcheck_all,		0 },
+	{ TKTFLAG_PROTECT_CFG2,		"PROTECT_CFG2",		vcheck_no_v1,		0 },
+	{ TKTFLAG_OATH_HOTP,		"OATH_HOTP",		vcheck_v21_or_greater,	0 },
+	{ TKTFLAG_CHAL_RESP,		"CHAL_RESP",		vcheck_v22_or_greater,	0 },
+	{ 0, "", 0 }
 };
 
 const char str_config_flags[] = "config_flags";
 struct map_st config_flags_map[] = {
-	{ CFGFLAG_SEND_REF, "SEND_REF", vcheck_all },
-	{ CFGFLAG_TICKET_FIRST, "TICKET_FIRST", vcheck_v1 },
-	{ CFGFLAG_PACING_10MS, "PACING_10MS", vcheck_all },
-	{ CFGFLAG_PACING_20MS, "PACING_20MS", vcheck_all },
-	{ CFGFLAG_ALLOW_HIDTRIG, "ALLOW_HIDTRIG", vcheck_v1 },
-	{ CFGFLAG_STATIC_TICKET, "STATIC_TICKET", vcheck_all },
-	{ CFGFLAG_SHORT_TICKET, "SHORT_TICKET", vcheck_no_v1 },
-	{ CFGFLAG_STRONG_PW1, "STRONG_PW1", vcheck_no_v1 },
-	{ CFGFLAG_STRONG_PW2, "STRONG_PW2", vcheck_no_v1 },
-	{ CFGFLAG_MAN_UPDATE, "MAN_UPDATE", vcheck_no_v1 },
-	{ CFGFLAG_OATH_HOTP8, "OATH_HOTP8", vcheck_v21_or_greater },
-	{ CFGFLAG_OATH_FIXED_MODHEX1, "OATH_FIXED_MODHEX1", vcheck_v21_or_greater },
-	{ CFGFLAG_OATH_FIXED_MODHEX2, "OATH_FIXED_MODHEX2", vcheck_v21_or_greater },
-	{ CFGFLAG_OATH_FIXED_MODHEX, "OATH_FIXED_MODHEX", vcheck_v21_or_greater },
+	/*
+	  Values used to pretty-print a YKP_CONFIG in ykp_write_config().
+
+	  The fourth field is a (tkt)context in which this (cfg)flag is valid.
+	  Some cfgFlags share the same value (e.g. CFGFLAG_STRONG_PW2 and
+	  CFGFLAG_OATH_FIXED_MODHEX2, both 0x40). Obvioulsy, STRONG_PW2 is not
+	  in effect when we do OATH, so by setting tkt_context to TKTFLAG_OATH_HOTP
+	  and having the OATH flags before STRONG_PW2 in this struct we will show
+	  cfgFlag 0x40 as OATH_FIXED_MODHEX2 and not STRONG_PW2 if TKTFLAG_OATH_HOTP
+	  is set.
+	*/
+	{ CFGFLAG_CHAL_YUBICO,		"CHAL_YUBICO",		vcheck_v22_or_greater,	TKTFLAG_CHAL_RESP },
+	{ CFGFLAG_CHAL_HMAC,		"CHAL_HMAC",		vcheck_v22_or_greater,	TKTFLAG_CHAL_RESP },
+	{ CFGFLAG_HMAC_LT64,		"HMAC_LT64",		vcheck_v22_or_greater,	TKTFLAG_CHAL_RESP },
+	{ CFGFLAG_CHAL_BTN_TRIG,	"CHAL_BTN_TRIG",	vcheck_v22_or_greater,	TKTFLAG_CHAL_RESP },
+	{ CFGFLAG_OATH_HOTP8,		"OATH_HOTP8",		vcheck_v21_or_greater,	TKTFLAG_OATH_HOTP },
+	{ CFGFLAG_OATH_FIXED_MODHEX1,	"OATH_FIXED_MODHEX1",	vcheck_v21_or_greater,	TKTFLAG_OATH_HOTP },
+	{ CFGFLAG_OATH_FIXED_MODHEX2,	"OATH_FIXED_MODHEX2",	vcheck_v21_or_greater,	TKTFLAG_OATH_HOTP },
+	{ CFGFLAG_OATH_FIXED_MODHEX,	"OATH_FIXED_MODHEX",	vcheck_v21_or_greater,	TKTFLAG_OATH_HOTP },
+	{ CFGFLAG_SEND_REF,		"SEND_REF",		vcheck_all,		0 },
+	{ CFGFLAG_TICKET_FIRST,		"TICKET_FIRST",		vcheck_v1,		0 },
+	{ CFGFLAG_PACING_10MS,		"PACING_10MS",		vcheck_all,		0 },
+	{ CFGFLAG_PACING_20MS,		"PACING_20MS",		vcheck_all,		0 },
+	{ CFGFLAG_ALLOW_HIDTRIG,	"ALLOW_HIDTRIG",	vcheck_v1,		0 },
+	{ CFGFLAG_STATIC_TICKET,	"STATIC_TICKET",	vcheck_all,		0 },
+	{ CFGFLAG_SHORT_TICKET,		"SHORT_TICKET",		vcheck_no_v1,		0 },
+	{ CFGFLAG_STRONG_PW1,		"STRONG_PW1",		vcheck_no_v1,		0 },
+	{ CFGFLAG_STRONG_PW2,		"STRONG_PW2",		vcheck_no_v1,		0 },
+	{ CFGFLAG_MAN_UPDATE,		"MAN_UPDATE",		vcheck_no_v1,		0 },
 	{ 0, "" }
 };
 
+const char str_extended_flags[] = "extended_flags";
+struct map_st extended_flags_map[] = {
+	{ EXTFLAG_SERIAL_BTN_VISIBLE,	"SERIAL_BTN_VISIBLE",	vcheck_v22_or_greater,	0 },
+	{ EXTFLAG_SERIAL_USB_VISIBLE,	"SERIAL_USB_VISIBLE",	vcheck_v22_or_greater,	0 },
+	{ EXTFLAG_SERIAL_API_VISIBLE,	"SERIAL_API_VISIBLE",	vcheck_v22_or_greater,	0 },
+	{ 0, "", 0 }
+};
+
 int ykp_write_config(const YKP_CONFIG *cfg,
 		     int (*writer)(const char *buf, size_t count,
 				   void *userdata),
@@ -380,29 +505,47 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 	if (cfg) {
 		char buffer[256];
 		struct map_st *p;
+		unsigned char t_flags;
+		bool key_bits_in_uid = false;
 
+		/* for OATH-HOTP and HMAC-SHA1 challenge response, there is four bytes
+		 *  additional key data in the uid field
+		 */
+		key_bits_in_uid = (_get_supported_key_length(cfg) == 20);
+
+		/* fixed: */
 		writer(str_fixed, strlen(str_fixed), userdata);
 		writer(str_key_value_separator,
 		       strlen(str_key_value_separator),
 		       userdata);
-		writer(str_modhex_prefix,
-		       strlen(str_key_value_separator),
-		       userdata);
-		yubikey_modhex_encode(buffer, (char *)cfg->ykcore_config.fixed, cfg->ykcore_config.fixedSize);
-		writer(buffer, strlen(buffer), userdata);
+		if (key_bits_in_uid) {
+			writer("n/a", 3, userdata);
+		} else {
+			writer(str_modhex_prefix,
+			       strlen(str_key_value_separator),
+			       userdata);
+			yubikey_modhex_encode(buffer, (char *)cfg->ykcore_config.fixed, cfg->ykcore_config.fixedSize);
+			writer(buffer, strlen(buffer), userdata);
+		}
 		writer("\n", 1, userdata);
 
+		/* uid: */
 		writer(str_uid, strlen(str_uid), userdata);
 		writer(str_key_value_separator,
 		       strlen(str_key_value_separator),
 		       userdata);
-		writer(str_hex_prefix,
-		       strlen(str_key_value_separator),
-		       userdata);
-		yubikey_hex_encode(buffer, (char *)cfg->ykcore_config.uid, UID_SIZE);
-		writer(buffer, strlen(buffer), userdata);
+		if (key_bits_in_uid) {
+			writer("n/a", 3, userdata);
+		} else {
+			writer(str_hex_prefix,
+			       strlen(str_key_value_separator),
+			       userdata);
+			yubikey_hex_encode(buffer, (char *)cfg->ykcore_config.uid, UID_SIZE);
+			writer(buffer, strlen(buffer), userdata);
+		}
 		writer("\n", 1, userdata);
 
+		/* key: */
 		writer(str_key, strlen(str_key), userdata);
 		writer(str_key_value_separator,
 		       strlen(str_key_value_separator),
@@ -411,9 +554,13 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 		       strlen(str_key_value_separator),
 		       userdata);
 		yubikey_hex_encode(buffer, (char *)cfg->ykcore_config.key, KEY_SIZE);
+		if (key_bits_in_uid) {
+			yubikey_hex_encode(buffer + KEY_SIZE * 2, (char *)cfg->ykcore_config.uid, 4);
+		}
 		writer(buffer, strlen(buffer), userdata);
 		writer("\n", 1, userdata);
 
+		/* acc_code: */
 		writer(str_acc_code, strlen(str_acc_code), userdata);
 		writer(str_key_value_separator,
 		       strlen(str_key_value_separator),
@@ -425,9 +572,10 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 		writer(buffer, strlen(buffer), userdata);
 		writer("\n", 1, userdata);
 
+		/* ticket_flags: */
 		buffer[0] = '\0';
 		for (p = ticket_flags_map; p->flag; p++) {
-			if (cfg->ykcore_config.tktFlags & p->flag
+			if ((cfg->ykcore_config.tktFlags & p->flag) == p->flag
 			    && p->vcheck(cfg)) {
 				if (*buffer) {
 					strcat(buffer, str_flags_separator);
@@ -444,9 +592,36 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 		writer(buffer, strlen(buffer), userdata);
 		writer("\n", 1, userdata);
 
+		/* config_flags: */
 		buffer[0] = '\0';
+		t_flags = cfg->ykcore_config.cfgFlags;
 		for (p = config_flags_map; p->flag; p++) {
-			if (cfg->ykcore_config.cfgFlags & p->flag
+			if ((t_flags & p->flag) == p->flag
+			    && p->vcheck(cfg)
+			    && (cfg->ykcore_config.tktFlags & p->tkt_context) == p->tkt_context) {
+				if (*buffer) {
+					strcat(buffer, str_flags_separator);
+					strcat(buffer, p->flag_text);
+				} else {
+					strcpy(buffer, p->flag_text);
+				}
+				/* make sure we don't show more than one cfgFlag per value -
+				   some cfgflags share value in different contexts
+				*/
+				t_flags -= p->flag;
+			}
+		}
+		writer(str_config_flags, strlen(str_config_flags), userdata);
+		writer(str_key_value_separator,
+		       strlen(str_key_value_separator),
+		       userdata);
+		writer(buffer, strlen(buffer), userdata);
+		writer("\n", 1, userdata);
+
+		/* extended_flags: */
+		buffer[0] = '\0';
+		for (p = extended_flags_map; p->flag; p++) {
+			if ((cfg->ykcore_config.extFlags & p->flag) == p->flag
 			    && p->vcheck(cfg)) {
 				if (*buffer) {
 					strcat(buffer, str_flags_separator);
@@ -456,7 +631,7 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 				}
 			}
 		}
-		writer(str_config_flags, strlen(str_config_flags), userdata);
+		writer(str_extended_flags, strlen(str_extended_flags), userdata);
 		writer(str_key_value_separator,
 		       strlen(str_key_value_separator),
 		       userdata);
@@ -467,6 +642,7 @@ int ykp_write_config(const YKP_CONFIG *cfg,
 	}
 	return 0;
 }
+
 int ykp_read_config(YKP_CONFIG *cfg,
 		    int (*reader)(char *buf, size_t count,
 				  void *userdata),
