@@ -62,6 +62,7 @@ int ykp_set_tktflag_APPEND_DELAY2(YKP_CONFIG *cfg, bool state);
 int ykp_set_tktflag_APPEND_CR(YKP_CONFIG *cfg, bool state);
 int ykp_set_tktflag_PROTECT_CFG2(YKP_CONFIG *cfg, bool state);
 int ykp_set_tktflag_OATH_HOTP(YKP_CONFIG *cfg, bool state);
+int ykp_set_tktflag_CHAL_RESP(YKP_CONFIG *cfg, bool state);
 
 int ykp_set_cfgflag_SEND_REF(YKP_CONFIG *cfg, bool state);
 int ykp_set_cfgflag_TICKET_FIRST(YKP_CONFIG *cfg, bool state);
@@ -77,6 +78,10 @@ int ykp_set_cfgflag_OATH_HOTP8(YKP_CONFIG *cfg, bool state);
 int ykp_set_cfgflag_OATH_FIXED_MODHEX1(YKP_CONFIG *cfg, bool state);
 int ykp_set_cfgflag_OATH_FIXED_MODHEX2(YKP_CONFIG *cfg, bool state);
 int ykp_set_cfgflag_OATH_FIXED_MODHEX(YKP_CONFIG *cfg, bool state);
+int ykp_set_cfgflag_CHAL_YUBICO(YKP_CONFIG *cfg, bool state);
+int ykp_set_cfgflag_CHAL_HMAC(YKP_CONFIG *cfg, bool state);
+int ykp_set_cfgflag_HMAC_LT64(YKP_CONFIG *cfg, bool state);
+int ykp_set_cfgflag_CHAL_BTN_TRIG(YKP_CONFIG *cfg, bool state);
 
 int ykp_write_config(const YKP_CONFIG *cfg,
 		     int (*writer)(const char *buf, size_t count,
